#ifndef _diff_myers_binary_diff_h_
#define _diff_myers_binary_diff_h_ 1

#include <limits.h>

#include "buf.h"
#include "internal/delta_int.h"
#include "internal/edit_script_int.h"

typedef struct storage_s {
        int *vec;
        int len;
        int max;
        int max_d;
} STORAGE_T;

typedef struct execution_s {
        BUF_T *a;
        BUF_T *b;
        EDIT_SCRIPT_T *script;
        int bailout_limit;
        STORAGE_T *storage;
} EXECUTION_T;

DELTA_RESULT_T myers_binary_diff(BUF_T *a, BUF_T *b, EDIT_SCRIPT_T *edit_script, int max_storage, int bailout_factor);
int myers_binary_diff_check_bounds(BUF_T *buf, int offset, int length);
int myers_binary_diff_corner(int d, int length);
int myers_binary_diff_bailout_limit(int bailout_factor, int n, int m);

// Does not take a copy of the contents of a, b or script.
EXECUTION_T *execution_create(BUF_T *a, BUF_T *b, EDIT_SCRIPT_T *script, int max_storage, int bailout_limit);
void execution_free(EXECUTION_T *execution);
DELTA_RESULT_T execution_diff(EXECUTION_T *execution, int aoff, int n, int boff, int m);
DELTA_RESULT_T execution_middlesnake(EXECUTION_T *execution, int aoff, int n, int boff, int m);
DELTA_RESULT_T execution_recurse(EXECUTION_T *execution, int aoff, int n, int boff, int m, int x, int u, int k);
DELTA_RESULT_T execution_bail(EXECUTION_T *execution, int d, int aoff, int n, int boff, int m);
DELTA_RESULT_T execution_bounded_diff(EXECUTION_T *execution, int aoff, int n, int boff, int m, int total_n, int total_m);

STORAGE_T *storage_create(int max);
void storage_free(STORAGE_T *storage);
void storage_initialise(STORAGE_T *storage, int d);
void storage_fill(STORAGE_T *storage, int start);
void storage_ensure(STORAGE_T *storage, int d);
int storage_extend(STORAGE_T *storage, int d);

int storage_key_forward(int k);
int storage_key_forward_get_next(STORAGE_T *storage, int k);
int storage_key_forward_get(STORAGE_T *storage, int k);
void storage_key_forward_set(STORAGE_T *storage, int k, int i);

int storage_key_reverse(int k);
int storage_key_reverse_get_next(STORAGE_T *storage, int k);
int storage_key_reverse_get(STORAGE_T *storage, int k);
void storage_key_reverse_set(STORAGE_T *storage, int k, int i);

#endif
